﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data;
using System.Data.SQLite;
using System.IO;
using smnetjs;

namespace arca4.Scripting.JS
{
    [SMEmbedded(AllowInheritedMembers = false)]
    class Sql : IDisposable
    {
        private String ConnectionString { get; set; }
        private SQLiteConnection Connection { get; set; }
        private SQLiteDataReader Reader { get; set; }

        public object this[string name]
        {
            get
            {
                this.lastError = String.Empty;

                if (this.Reader == null)
                {
                    this.lastError = "no data available";
                    return null;
                }

                try
                {
                    return this.Reader[name];
                }
                catch (Exception e)
                {
                    this.lastError = e.Message;
                    return null;
                }
            }
        }

        public String lastError { get; private set; }

        public Sql(String database)
        {
            if (String.IsNullOrEmpty(database))
                throw new Exception("invalid database name"); // sandboxed file path

            foreach (char c in Path.GetInvalidFileNameChars())
                if (database.Contains(c))
                    throw new Exception("invalid database name"); // sandboxed file path

            this.lastError = String.Empty;

            String path = Environment.GetFolderPath(System.Environment.SpecialFolder.ApplicationData);
            path = Path.Combine(path, "Arca4", "Sql");

            if (!Directory.Exists(path))
                Directory.CreateDirectory(path);

            String file = Path.Combine(path, database);

            if (Path.GetDirectoryName(file) == path)
            {
                this.ConnectionString = "Data Source=" + file;

                if (!File.Exists(file))
                    SQLiteConnection.CreateFile(file);
            }
            else throw new Exception("invalid database name"); // sandboxed file path
        }

        public bool connected
        {
            get
            {
                if (this.Connection != null)
                    return this.Connection.State == ConnectionState.Open;

                return false;
            }
        }

        public bool open()
        {
            this.lastError = String.Empty;

            try
            {
                if (this.Reader != null)
                {
                    this.Reader.Close();
                    this.Reader.Dispose();
                    this.Reader = null;
                }
            }
            catch { }

            try
            {
                if (this.Connection == null)
                    this.Connection = new SQLiteConnection(this.ConnectionString);

                if (this.Connection.State != ConnectionState.Open)
                    this.Connection.Open();

                return this.Connection.State == ConnectionState.Open;
            }
            catch { return false; }
        }

        public void close()
        {
            this.lastError = String.Empty;

            try
            {
                if (this.Connection != null)
                {
                    this.Connection.Close();
                    this.Connection.Dispose();
                    this.Connection = null;
                }
            }
            catch { }

            try
            {
                if (this.Reader != null)
                {
                    this.Reader.Close();
                    this.Reader.Dispose();
                    this.Reader = null;
                }
            }
            catch { }
        }

        public bool getRows(SqlQuery query)
        {
            this.lastError = String.Empty;

            try
            {
                if (this.Reader != null)
                {
                    this.Reader.Close();
                    this.Reader.Dispose();
                    this.Reader = null;
                }
            }
            catch { }

            if (!this.connected)
            {
                this.lastError = "connection closed";
                return false;
            }

            try
            {
                using (SQLiteCommand cmd = new SQLiteCommand(query.query, this.Connection))
                {
                    for (int i = 0; i < query.length; i++)
                        cmd.Parameters.Add(query[i]);

                    this.Reader = cmd.ExecuteReader();
                }
            }
            catch (Exception e)
            {
                this.lastError = e.Message;
                return false;
            }

            return true;
        }

        public bool canRead
        {
            get
            {
                if (this.Reader == null)
                    return false;

                if (!this.Reader.Read())
                    return false;

                return true;
            }
        }

        public bool exec(SqlQuery query)
        {
            this.lastError = String.Empty;

            try
            {
                if (this.Reader != null)
                {
                    this.Reader.Close();
                    this.Reader.Dispose();
                    this.Reader = null;
                }
            }
            catch { }

            if (!this.connected)
            {
                this.lastError = "connection closed";
                return false;
            }

            try
            {
                using (SQLiteCommand cmd = new SQLiteCommand(query.query, this.Connection))
                {
                    for (int i = 0; i < query.length; i++)
                        cmd.Parameters.Add(query[i]);

                    cmd.ExecuteNonQuery();
                }

                return true;
            }
            catch (Exception e) { this.lastError = e.Message; }

            return false;
        }

        [SMIgnore]
        public void Dispose()
        {
            try
            {
                if (this.Connection != null)
                {
                    this.Connection.Close();
                    this.Connection.Dispose();
                    this.Connection = null;
                }
            }
            catch { }

            try
            {
                if (this.Reader != null)
                {
                    this.Reader.Close();
                    this.Reader.Dispose();
                    this.Reader = null;
                }
            }
            catch { }
        }
    }
}
