﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Diagnostics.Contracts;
using System.Reflection;
using smnetjs;

namespace arca4.Scripting
{
	sealed class ScriptManager
	{

		#region Instance Based Script Manager Class

		public sealed class ScriptManagerIndexer : IEnumerable<ScriptObj>
		{
			#region Class Members

			private List<ScriptObj> scripts = null;

			#endregion

			#region Constructors 

			public ScriptManagerIndexer()
			{
				scripts = new List<ScriptObj>();
			}

			#endregion

			#region Indexer and Enumerator Methods / Properties

			public ScriptObj this[int index]
			{
				get
				{
					if (scripts.Count < index)
						throw new ArgumentOutOfRangeException("index");

					return scripts[index];
				}
			}

			public ScriptObj this[string scriptName]
			{
				get
				{

					Contract.Requires(!String.IsNullOrEmpty(scriptName));
					Contract.Ensures(Contract.Result<ScriptObj>() != null);

					ScriptObj script = scripts.Where(c => scriptName.Equals(c.ScriptName)).First();

					return script;

				}
			}

			public IEnumerator<ScriptObj> GetEnumerator()
			{
				return scripts.GetEnumerator();
			}

			IEnumerator IEnumerable.GetEnumerator()
			{
				return scripts.GetEnumerator();
			}

			#endregion

			#region Properties

			public int Count { get { return scripts.Count; } }

			#endregion

			#region Public Methods

			public void Add(string scriptName, string scriptText = null)
			{
				Contract.Requires(string.IsNullOrWhiteSpace(scriptName) == false);

				if (Find(scriptName) != null)
					Remove(scriptName);

				ScriptObj script = new ScriptObj(scriptName, scriptText);
				scripts.Add(script);
				script.Execute();
			}

			public void Remove(string scriptName)
			{
				Contract.Requires(string.IsNullOrWhiteSpace(scriptName) == false);

				if (scripts.Where(c => scriptName.Equals(c.ScriptName)).Count() == 0)
					return;

				EventManager.UnRegisterEvents(scriptName);

				this[scriptName].Unload();
				
				scripts.Remove(this[scriptName]);

                ScriptTimers.KillTimers(scriptName); // oobe: destroy any pending timers created in this script
			}

			public ScriptObj Find(string scriptName)
			{
				Contract.Requires(string.IsNullOrWhiteSpace(scriptName) == false);

				return scripts.Find(c => c.ScriptName.Equals(scriptName, StringComparison.InvariantCultureIgnoreCase));
			}

			#endregion

		}

		#endregion

		#region Static Members

		private static SMRuntime runtime = null;

		#endregion

		#region Static Constructor

		static ScriptManager()
		{			
			runtime = new SMRuntime(RuntimeOptions.DisableAutoEmbedding | RuntimeOptions.ForceGCOnContextDestroy);
						
			bool ret = runtime.Embed(typeof(EventType));
			ret &= runtime.Embed(typeof(RejectionType));

			foreach (Type t in Assembly.GetExecutingAssembly().GetTypes())
			{
				if (t.Namespace == "arca4.Scripting.JS")
				{
					ret &= runtime.Embed(t);
				}
			}

			runtime.OnScriptError += new ScriptErrorHandler(delegate(SMScript script, SMErrorReport report)
			{
				DebugLog.WriteLine(
				string.Format("error: {0} in {1} on line {2}", report.Message, script.Name, report.LineNumber));
			});
		}

		#endregion

		#region Static Properties

		internal static SMRuntime Runtime { get { return runtime; } }

		#endregion

		#region Static Indexer Stuff / Hack

		public static ScriptManagerIndexer Items
		{
			get { return staticIndexer ?? (staticIndexer = new ScriptManagerIndexer()); }
		}

		private static ScriptManagerIndexer staticIndexer;

		#endregion

        // oobe

        

        public static void AutoLoad()
        {
            String script = null;

            try
            {
                String path = Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData);
                path = Path.Combine(path, "Arca4", "Scripts");

                if (!Directory.Exists(path))
                    return;

                path = Path.Combine(path, "autorun.js");

                if (!File.Exists(path))
                    return;

                script = File.ReadAllText(path);
            }
            catch { }

            if (script != null)
            {
                Items.Add("autorun.js", script);
                Items.Remove("autorun.js");
            }
        }

	}
}
