﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Xml;
using System.Net;
using System.Xml.Linq;
using arca4;
using Compression;

namespace Ares.UDP
{
    class UdpServerPool
    {
        private static Random rnd;
        public static List<UdpServerObject> Pool;
        public static UdpSortMethod SortMethod;

        public static void AddServer(UdpServerObject server)
        {
            if (server.ip.Equals(Settings.ExternalIP)) // don't want our own room on the list since that will be sent anyway
                return;

            UdpServerObject sobj = Pool.Find(s => s.ip.Equals(server.ip));

            if (sobj != null) // maybe room port has changed?
            {
                sobj.port = server.port;
                return;
            }

            if (Pool.Count > 4000)
                PurgeList();

            Pool.Add(server);
        }

        public static void ExpireList()
        {
            uint time = Helpers.UnixTime;
            SortMethod = UdpSortMethod.SortMostRecentFirst;
            Pool.Sort();
            Pool.RemoveAll(s => s.try_count > 4 && (time - s.last_sent_add_IPS) > 60 && (time - s.last_connect) > 3600);
        }

        public static void PurgeList()
        {
            SortMethod = UdpSortMethod.SortMostRecentFirst;
            Pool.Sort();

            uint time = Helpers.UnixTime;
            int never_counted = Pool.FindAll(s => s.last_sent_add_IPS == 0).Count;
            int successful = Pool.FindAll(s => (s.last_connect + 900) > time).Count;

            Pool.Reverse();

            if (never_counted <= 600 && successful >= 50)
            {
                int target = Pool.FindIndex(s => s.last_sent_add_IPS == 0);

                if (target > -1)
                {
                    Pool.RemoveAt(target);
                    return;
                }
            }

            SortMethod = UdpSortMethod.SortMostAckedFirst;
            Pool.Sort();
            Pool.RemoveAt(Pool.Count - 1);
        }

        public static UdpServerObject[] GetServers(int max_servers)
        {
            ShuffleList();

            uint time = Helpers.UnixTime;
            List<UdpServerObject> results = Pool.FindAll(s => s.ack_count > 0 && (s.last_connect + 900) > time);

            if (results.Count > max_servers)
                results = results.GetRange(0, max_servers);

            return results.ToArray();
        }

        public static UdpServerObject[] GetServers(IPAddress target_ip, int max_servers)
        {
            ShuffleList();

            uint time = Helpers.UnixTime;
            List<UdpServerObject> results = Pool.FindAll(s => !s.ip.Equals(target_ip) && s.ack_count > 0 && (s.last_connect + 900) > time);

            if (results.Count > max_servers)
                results = results.GetRange(0, max_servers);

            return results.ToArray();
        }

        public static void Load()
        {
            Pool = new List<UdpServerObject>();
            SortMethod = UdpSortMethod.SortMostRecentFirst;

            String path = Environment.GetFolderPath(System.Environment.SpecialFolder.ApplicationData);
            path = Path.Combine(path, "Arca4");

            if (!Directory.Exists(path))
                Directory.CreateDirectory(path);

            path = Path.Combine(path, "udp.xml");

            if (File.Exists(path))
            {
                try
                {
                    XDocument xml = XDocument.Load(path);

                    Pool = (from i in xml.Descendants("Server")
                            select new UdpServerObject
                            {
                                ip = IPAddress.Parse(i.Element("ExternalIP").Value),
                                port = ushort.Parse(i.Element("Port").Value),
                                ack_count = int.Parse(i.Element("AckCount").Value)
                            }).ToList<UdpServerObject>();
                }
                catch { }
            }

            if (Pool.Count < 5)
            {
                DebugLog.WriteLine("downloading remote udp node cache");
                Pool.Clear();
                Download();
            }
            else DebugLog.WriteLine("loaded " + Pool.Count + " nodes locally");

            DebugLog.WriteLine("beginning firewall trial");
        }

        public static void Save()
        {
            if (Settings.CanList)
            {
                XDocument xml = new XDocument(new XElement("Udp"));

                SortMethod = UdpSortMethod.SortMostRecentFirst;
                Pool.Sort();

                int counter = 0;
                int time = (int)Helpers.UnixTime;

                foreach (UdpServerObject i in Pool)
                {
                    if (i.ack_count > 0 && (time - i.last_connect) < 1800)
                    {
                        if (i.ack_count > 65000)
                            i.ack_count = 40000;

                        xml.Element("Udp").Add(new XElement("Server",
                            new XElement("ExternalIP", i.ip),
                            new XElement("Port", i.port),
                            new XElement("AckCount", i.ack_count)));

                        if (counter++ == 100)
                            break;
                    }
                }

                if (counter > 25)
                {
                    String path = Environment.GetFolderPath(System.Environment.SpecialFolder.ApplicationData);
                    path = Path.Combine(path, "Arca4");

                    if (!Directory.Exists(path))
                        Directory.CreateDirectory(path);

                    path = Path.Combine(path, "udp.xml");

                    try
                    {
                        xml.Save(path);
                    }
                    catch { }

                    DebugLog.WriteLine("udp node list updated - [" + UdpStats.ClientRequests + ":" +
                        UdpStats.ClientResponses + ":" + UdpStats.PushAttempts + ":" + UdpStats.SuccessfulPushes + "]");
                }
                else
                {
                    DebugLog.WriteLine("udp node list expired - downloading remote udp node cache");
                    Pool.Clear();
                    Download();
                }

                UdpStats.Reset();
            }
        }

        public static void ShuffleList()
        {
            if (rnd == null)
                rnd = new Random();

            int n = Pool.Count;

            while (n > 1)
            {
                n--;
                int k = rnd.Next(n + 1);
                UdpServerObject value = Pool[k];
                Pool[k] = Pool[n];
                Pool[n] = value;
            }
        }

        private static void Download()
        {
            try
            {
                WebRequest request = WebRequest.Create("http://chatrooms.marsproject.net/rooms.aspx");
                request.Timeout = 8000;
                WebResponse response = request.GetResponse();
                List<byte> list = new List<byte>();

                using (Stream stream = response.GetResponseStream())
                {
                    int size = 0;
                    byte[] buffer = new byte[8192];

                    while ((size = stream.Read(buffer, 0, 8192)) > 0)
                        list.AddRange(buffer.Take(size));
                }

                response.Close();

                ushort unpacked_size = BitConverter.ToUInt16(list.ToArray(), 0);
                list.RemoveRange(0, 2);
                byte[] unpacked = list.ToArray();
                unpacked = ZipLib.Decompress(unpacked, unpacked_size);
                list = new List<byte>(unpacked);

                while (list.Count >= 6)
                {
                    IPAddress ip = new IPAddress(list.GetRange(0, 4).ToArray());
                    ushort port = BitConverter.ToUInt16(list.ToArray(), 4);
                    list.RemoveRange(0, 6);

                    UdpServerObject obj = new UdpServerObject();
                    obj.ip = ip;
                    obj.port = port;
                    obj.ack_count = 10;

                    Pool.Add(obj);
                }

                XDocument xml = new XDocument(new XElement("Udp"));

                foreach (UdpServerObject i in Pool)
                    xml.Element("Udp").Add(new XElement("Server",
                                new XElement("ExternalIP", i.ip),
                                new XElement("Port", i.port),
                                new XElement("AckCount", i.ack_count)));

                String path = Environment.GetFolderPath(System.Environment.SpecialFolder.ApplicationData);
                path = Path.Combine(path, "Arca4");

                if (!Directory.Exists(path))
                    Directory.CreateDirectory(path);

                path = Path.Combine(path, "udp.xml");
                xml.Save(path);

                DebugLog.WriteLine("remote udp node cache download successfully");
            }
            catch { DebugLog.WriteLine("remote udp node cache download failed, trying again in 15 minutes"); }

        }

    }
}
